-- SQL2016+ ... use string_split function (quite efficient).
declare @testString nvarchar(1000) = '23310,Arnold,Powers,5 SW Kahl St,Springfield,TN';
declare @separator nvarchar(10) = ',';

select value FieldValue
from string_split(@testString, @separator);
go

-- Use alternative methods pre-2016.
-- Less efficient, but one advantage of this method is guaranteed order.
declare @testString nvarchar(1000) = '23310,Arnold,Powers,5 SW Kahl St,Springfield,TN';
declare @separator nvarchar(10) = ',';
declare @testStringPlus nvarchar(1001) = @testString + ',';

with SeparatorLocations as
(
	select n.n
	from Admin.dbo.fnGetNums(len(@testStringPlus)) n
	where substring(@testStringPlus, n.n, 1) = @separator
), StartEndPositions as
(
	select lag(sl.n, 1, 0) over (order by sl.n) + 1 StartPos, sl.n EndPos
	from SeparatorLocations sl
)
select substring(@testString, sep.StartPos, sep.EndPos - sep.StartPos) FieldValue
from StartEndPositions sep
order by StartPos;
go

-- XML method
declare @testString nvarchar(1000) = '23310,Arnold,Powers,5 SW Kahl St,Springfield,TN';
declare @separator nvarchar(10) = ',';
declare @xml xml = cast(('<delimiter>' + replace(@testString, @separator, '</delimiter><delimiter>') + '</delimiter>') as xml);
select n.value('.', 'varchar(50)') as FieldValue from @xml.nodes('delimiter') t(n);

-- Test efficiency
-- Using string_split
declare @testString nvarchar(max) = (select top 1 NumberList from Admin.dbo.NumberString);
declare @separator nvarchar(10) = ',';

select value FieldValue
from string_split(@testString, @separator);
go

-- Using row_number
declare @testString nvarchar(max) = (select top 1 NumberList from Admin.dbo.NumberString);
declare @separator nvarchar(10) = ',';
declare @testStringPlus nvarchar(max) = @testString + ',';

with SeparatorLocations as
(
	select n.n
	from Admin.dbo.fnGetNums(len(@testStringPlus)) n
	where substring(@testStringPlus, n.n, 1) = @separator
), StartEndPositions as
(
	select lag(sl.n, 1, 0) over (order by sl.n) + 1 StartPos, sl.n EndPos
	from SeparatorLocations sl
)
select substring(@testString, sep.StartPos, sep.EndPos - sep.StartPos) FieldValue
from StartEndPositions sep
order by StartPos;
go

-- XML
declare @testString nvarchar(max) = (select top 1 NumberList from Admin.dbo.NumberString);
declare @separator nvarchar(10) = ',';
declare @xml xml = cast(('<delimiter>' + replace(@testString, @separator, '</delimiter><delimiter>') + '</delimiter>') as xml);
select n.value('.', 'varchar(50)') as FieldValue from @xml.nodes('delimiter') t(n);
