use CorpDB;
go

-- Get 100 random customers
with CustomersRandomized as
(
	select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State,
		   row_number() over (order by newid()) rn
	from dbo.Customer c
)
select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State
from CustomersRandomized c
where c.rn <= 100;

-- Get 10 random customers from each state
with CustomersRandomized as
(
	select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State,
		   row_number() over (partition by c.State order by newid()) rn
	from dbo.Customer c
)
select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State
from CustomersRandomized c
where c.rn <= 10
order by c.State;

-- Get x random customers per state, where "x" is proportional to the number of customers from that state; approximately 500 total
with CustomersRandomized as
(
	select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State,
		   ceiling(500.0 * count(c.CustomerID) over (partition by c.State) / count(c.CustomerID) over ()) PersonsToSample,
		   row_number() over (partition by c.State order by newid()) rn
	from dbo.Customer c
)
select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State
from CustomersRandomized c
where c.rn <= PersonsToSample
order by c.State;

-- Get x random customers per state, where "x" is proportional to the number of customers from that state; approximately 500 total
with CustomersRandomized as
(
	select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State,
		   500.0 * count(c.CustomerID) over (partition by c.State) / count(c.CustomerID) over () PersonsToSample,
		   row_number() over (partition by c.State order by newid()) rn
	from dbo.Customer c
)
select c.CustomerID,
		   c.FirstName,
		   c.LastName,
		   c.Address,
		   c.City,
		   c.State
from CustomersRandomized c
where c.rn <= case when c.PersonsToSample < 1 then 1 else c.PersonsToSample end
order by c.State;
