use CorpDB;
go

drop table if exists #PrimeCandidate;
go

create table #PrimeCandidate
(
	Prime int not null
);
go
create unique clustered index pk_PrimeCandidate on #PrimeCandidate (Prime);
go

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5)
insert #PrimeCandidate (Prime)
select n.num as CustomerID
from nums n
where n.num >= 2
and n.num <= 1000000
order by n.num;
go

declare @rangeLow int = (select min(Prime) from #PrimeCandidate);
declare @rangeHigh int;
declare @rangeSize int = 1000;
declare @tableMax int = (select max(Prime) from #PrimeCandidate);

while @rangeLow < @tableMax
begin
	select @rangeHigh =
		case
			when @rangeLow + @rangeSize < @tableMax + 1
			then @rangeLow + @rangeSize
			else @tableMax + 1
		end;

		with TestNumbers as
		(
			select pc.Prime
			from #PrimeCandidate pc
			where pc.Prime <= sqrt(@rangeHigh)
		), RangeNumbers as
		(
			select pc.Prime
			from #PrimeCandidate pc
			where pc.Prime >= @rangeLow
			and pc.Prime < @rangeHigh
		)
		delete rn
		from RangeNumbers rn
		where exists
		(
			select *
			from TestNumbers tn
			where tn.Prime < rn.Prime
			and rn.Prime % tn.Prime = 0
		);

	select @rangeLow = @rangeHigh;
	--select @rangeLow = @tableMax;
end
go

drop table if exists PrimeNumber;
go

create table PrimeNumber
(
	Prime int not null
);
go

insert dbo.PrimeNumber (Prime)
select Prime
from #PrimeCandidate;
