----------------------------------------------------------------------------------------------------
-- Sample of how to get a short hash, of sorts.
----------------------------------------------------------------------------------------------------
select left(convert(nvarchar(100), hashbytes('SHA1', lower('tf3604')), 2), 10);

----------------------------------------------------------------------------------------------------
-- Build list of common email host servers.
----------------------------------------------------------------------------------------------------
if object_id('Work.dbo.CommonHosts') is not null
	drop table Work.dbo.CommonHosts;
go
create table Work.dbo.CommonHosts
(
	HostName nvarchar(100)
);

insert Work.dbo.CommonHosts (HostName)
values ('yahoo.com'), ('gmail.com'), ('hotmail.com'), ('aol.com'), ('comcast.net'), ('msn.com'), ('sbcglobal.net'),
	('verizon.net'), ('live.com'), ('bellsouth.net'), ('att.net'), ('earthlink.net'), ('cox.net'), ('juno.com'),
	('optonline.net'), ('ymail.com'), ('cs.com'), ('mac.com'), ('charter.net'), ('me.com');

----------------------------------------------------------------------------------------------------
-- Obfuscate those email addresses.
----------------------------------------------------------------------------------------------------
with ProperAddress as
(
	select	e.ID,
			e.Address
	from	Work.dbo.Email e
	where	e.Address like '%@%.%'
), ImproperAddress as
(
	select	e.ID,
			e.Address
	from	Work.dbo.Email e
	where	e.Address not like '%@%.%'
), OffsetInfo as
(
	select	pa.ID,
			pa.Address,
			1 UserStart,
			charindex('@', pa.Address) - 1 UserLength,
			'@' UserSeparator,
			charindex('@', pa.Address) + 1 HostStart,
			charindex('.', pa.Address, charindex('@', pa.Address)) - charindex('@', pa.Address) - 1 HostLength,
			'.' HostSeparator,
			charindex('.', pa.Address, charindex('@', pa.Address)) + 1 TLDStart,
			len(pa.Address) - charindex('.', pa.Address, charindex('@', pa.Address)) TLDLength
	from	ProperAddress pa

	union all

	select	ia.ID,
			ia.Address,
			1 UserStart,
			len(ia.Address) UserLength,
			'' UserSeparator,
			1 HostStart,
			0 HostLength,
			'' HostSeparator,
			1 TLDStart,
			0 TLDLength
	from	ImproperAddress ia
), Parts as
(
	select	oi.ID,
			oi.Address,
			substring(oi.Address, oi.UserStart, oi.UserLength) UserName,
			oi.UserSeparator,
			substring(oi.Address, oi.HostStart, oi.HostLength) HostName,
			oi.HostSeparator,
			substring(oi.Address, oi.TLDStart, oi.TLDLength) TLD
	from	OffsetInfo oi
), ObfuscatedParts as
(
	select		p.ID,
				p.Address OriginalAddress,
				left(convert(nvarchar(100), hashbytes('SHA1', lower(p.UserName)), 2), 10) +
					p.UserSeparator +
					case when ch.HostName is not null
						then ch.HostName
						else left(convert(nvarchar(100), hashbytes('SHA1', lower(p.HostName)), 2), 10) +
							p.HostSeparator +
							p.TLD
					end Address
	from		Parts p
	left join	Work.dbo.CommonHosts ch on ch.HostName = p.HostName + p.HostSeparator + p.TLD
)
select top 100 *
from ObfuscatedParts;

----------------------------------------------------------------------------------------------------
-- Generate random character strings
----------------------------------------------------------------------------------------------------
if object_id('tempdb.dbo.#RandomChars') is not null
	drop table #RandomChars;
create table #RandomChars (StringNumber int, Position int, rnd int);

if object_id('tempdb.dbo.#Chars') is not null
	drop table #Chars;
create table #Chars (ID int not null identity(1,1), c char(1) not null);

insert #Chars (c) values 
	('a'), ('b'), ('c'), ('d'), ('e'), ('f'), ('g'), ('h'), ('i'), ('j'), ('k'), ('l'), ('m'),
	('n'), ('o'), ('p'), ('q'), ('r'), ('s'), ('t'), ('u'), ('v'), ('w'), ('x'), ('y'), ('z');

declare @charCount int = (select count(*) from #Chars);
declare @stringLen int = 10;
declare @stringCount int = 1000;

insert #RandomChars (StringNumber, Position, rnd)
select strnum.n, pos.n, cast(((binary_checksum(newid()) + 2147483648.) / 4294967296.) * @charCount + 1 as int) rnd
from Admin.dbo.fnGetNums(@stringCount) strnum
cross apply Admin.dbo.fnGetNums(@stringLen) pos;

--select r.StringNumber, r.Position, c.c
--from #RandomChars r
--inner join #Chars c on r.rnd = c.ID
--order by r.StringNumber, r.Position;

--with RandomChars as
--(
--	select r.StringNumber, r.Position, c.c
--	from #RandomChars r
--	inner join #Chars c on r.rnd = c.ID
--), Strings as
--(
--	select distinct r.StringNumber
--	from #RandomChars r
--)
--select s.StringNumber,
--	c1.c + c2.c + c3.c + c4.c + c5.c + c6.c + c7.c + c8.c + c9.c + c10.c String
--from Strings s
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 1) c1
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 2) c2
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 3) c3
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 4) c4
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 5) c5
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 6) c6
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 7) c7
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 8) c8
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 9) c9
--cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = 10) c10;

declare @concat nvarchar(max) = null;
declare @crossApply nvarchar(max) = '';

select @concat = isnull(@concat + ' + ', '') + 'c' + cast(n.n as nvarchar(20)) + '.c',
	@crossApply = @crossApply + 'cross apply (select r.c from RandomChars r where r.StringNumber = s.StringNumber and r.Position = ' + cast(n.n as nvarchar(20)) + ') c' + cast(n.n as nvarchar(20)) + '
'
from Admin.dbo.fnGetNums(@stringLen) n;

declare @sql nvarchar(max) = 'with RandomChars as
(
	select r.StringNumber, r.Position, c.c
	from #RandomChars r
	inner join #Chars c on r.rnd = c.ID
), Strings as
(
	select distinct r.StringNumber
	from #RandomChars r
)
select s.StringNumber, ' + @concat + ' String
from Strings s
' + @crossApply + ';';

exec(@sql);
