use Admin;
GO

-- truncate table dbo.BackupLsnMapping

if not exists (select * from sys.tables t inner join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'BackupLsnMapping')
begin
	create table dbo.BackupLsnMapping
	(
		MappingId int identity(1,1) not null,
		OriginalBackupId int not null,
		ServerName nvarchar(256) not null,
		DatabaseName nvarchar(256) not null,
		BackupType varchar(5) not null,
		LocalFileName nvarchar(400) not null,
		SharedFileName nvarchar(400) not null,
		FileTime datetime2(7) null,
		FileSize bigint null,
		BackupStartDate datetime2(7) null,
		BackupEndDate datetime2(7) null,
		FirstLSN numeric(25, 0) null,
		LastLSN numeric(25, 0) null,
		DifferentialLSN numeric(25, 0) null,
		ErrorCount int not null default(0),
		CollectionTime datetime2(7) not null default (sysdatetime()),
		constraint pk_BackupLsnMapping primary key clustered (MappingId)
	);
end
if not exists (select * from sys.tables t inner join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'BackupLsnErrorMessage')
begin
	create table dbo.BackupLsnErrorMessage
	(
		ErrorId int identity(1,1) not null,
		MappingId int not null,
		ErrorTime datetime2(7) not null default(sysdatetime()),
		ErrorNumber int null,
		ErrorSeverity int null,
		ErrorState int null,
		ErrorMessage nvarchar(max) null,
		constraint pk_BackupLsnErrorMessage primary key clustered (ErrorId),
		constraint fk_BackupLsnErrorMessage__MappingId foreign key (MappingId) references dbo.BackupLsnMapping (MappingId)
	);
end
GO

insert dbo.BackupLsnMapping (OriginalBackupId, ServerName, DatabaseName, LocalFileName, SharedFileName, BackupType)
select		bs.backup_set_id OriginalBackupId,
			@@ServerName ServerName, 
			bs.database_name DatabaseName, 
			bmf.physical_device_name LocalFileName,
			cast('\\' + case when charindex('\', @@ServerName) = 0 then @@ServerName else left(@@ServerName, charindex('\', @@ServerName) - 1) end + 
				'\' + replace(bmf.physical_device_name, ':\', '$\') as nvarchar(256)) SharedFileName,
			case bs.type when 'D' then 'Full' when 'I' then 'Diff' when 'L' then 'Log' end BackupType
from msdb.dbo.backupmediafamily bmf
inner join msdb.dbo.backupset bs on bmf.media_set_id = bs.media_set_id 
left join dbo.BackupLsnMapping map on map.OriginalBackupId = bs.backup_set_id
where bs.type in ('D', 'I', 'L') -- Full, Diff, Log
and bs.backup_set_id > (select coalesce(max(OriginalBackupId), 0) from dbo.BackupLsnMapping)
and bmf.physical_device_name not like '{[0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F]-[0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F]-[0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F]-[0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F]-[0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F][0-9,A-F]}%' -- skip GUID names (operating system-level backups)
and map.MappingId is null;
