use Admin;
go

if OBJECT_ID('tempdb..#hdr') is not null
	drop table #hdr;

create table #hdr
(
	BackupName				nvarchar(128),		
	BackupDescription		nvarchar(256),		
	BackupType				smallint,		
	ExpirationDate			datetime,		
	Compressed				bit,		
	Position				smallint,		
	DeviceType				tinyint,		
	UserName				nvarchar(128),		
	ServerName				nvarchar(128),		
	DatabaseName			nvarchar(128),		
	DatabaseVersion			int,		
	DatabaseCreationDate	datetime,		
	BackupSize				numeric(20,0),		
	FirstLsn				numeric(25,0),		
	LastLsn					numeric(25,0),		
	CheckpointLsn			numeric(25,0),		
	DatabaseBackupLsn		numeric(25,0),		
	BackupStartDate			datetime,		
	BackupFinishDate		datetime,		
	SortOrder				smallint,		
	CodePage				smallint,		
	UnicodeLocaleId			int,
	UnicodeComparisonStyle	int,
	CompatibilityLevel		tinyint,		
	SoftwareVendorId		int,		
	SoftwareVersionMajor	int,		
	SoftwareVersionMinor	int,		
	SoftwareVersionBuild	int,		
	MachineName				nvarchar(128),		
	Flags					int,
	BindingId				uniqueidentifier,		
	RecoveryForkId			uniqueidentifier,		
	Collation				nvarchar(128),
	FamilyGUID				uniqueidentifier,
	HasBulkLoggedData		bit,
	IsSnapshot				bit,
	IsReadOnly				bit,
	IsSingleUser			bit,
	HasBackupChecksums		bit,
	IsDamaged				bit,
	BeginsLogChain			bit,
	HasIncompleteMetaData	bit,
	IsForceOffline			bit,
	IsCopyOnly				nvarchar(256),
	FirstRecoveryForkID		uniqueidentifier,
	ForkPointLsn			numeric(25,0),
	RecoveryModel			nvarchar(60),
	DifferentialBaseLsn		numeric(25,0),
	DifferentialBaseGUID	uniqueidentifier,
	BackupTypeDescription	nvarchar(60),
	BackupSetGUID			uniqueidentifier,
	CompressedBackupSize	bigint,
	Containment				tinyint
);

declare	@MappingId int = null,
		@BackupFileName nvarchar(256) = null,
		@FirstLSN numeric(25,0),
		@LastLSN numeric(25,0),
		@DifferentialLSN numeric(25,0);

select		top 1
			@MappingId = MappingId,
			@BackupFileName = LocalFileName
from		dbo.BackupLsnMapping
where		FirstLSN is null
and			ErrorCount < 5
order by	MappingId;

while @MappingId is not null
begin
	truncate table #hdr;

	declare @sql nvarchar(max) = N'restore headeronly from disk = ''' + @BackupFileName + '''';
	
	begin try
		insert		#hdr
		exec		(@sql);
	
		select		@FirstLSN = null,
					@LastLSN = null,
					@DifferentialLSN = null;
				
		select		@FirstLSN = FirstLsn,
					@LastLSN = LastLsn,
					@DifferentialLSN = DifferentialBaseLsn
		from		#hdr;
	
		update		dbo.BackupLsnMapping
		set			FirstLSN = hdr.FirstLsn,
					LastLSN = hdr.LastLsn,
					DifferentialLSN = hdr.DifferentialBaseLsn,
					BackupStartDate	= hdr.BackupStartDate,
					BackupEndDate = hdr.BackupFinishDate,
					ErrorCount = 0
		from		#hdr hdr
		where		LocalFileName = @BackupFileName;
	end try
	begin catch
		update		dbo.BackupLsnMapping
		set			ErrorCount = ErrorCount + 1
		where		MappingId = @MappingId;

		insert		dbo.BackupLsnErrorMessage (MappingId, ErrorNumber, ErrorSeverity, ErrorState, ErrorMessage)
		values		(@MappingId, error_number(), error_severity(), error_state(), error_message());
	end catch

	declare		@PreviousMappingId int = @MappingId;
	
	select		@MappingId = null,
				@BackupFileName = null;
	
	select		top 1
				@MappingId = MappingId,
				@BackupFileName = LocalFileName
	from		dbo.BackupLsnMapping
	where		FirstLSN is null
	and			MappingId > @PreviousMappingId
	and			ErrorCount < 5
	order by	MappingId;
end
