if object_id('tempdb.dbo.#BaseballStandings') is not null
	drop table #BaseballStandings;
go

-- MLB 2018 season final results
create table #BaseballStandings
(
	TeamName nvarchar(75) not null,
	League nvarchar(10) not null,
	Division nvarchar(10) not null,
	Wins int not null,
	Losses int not null
);

insert #BaseballStandings (TeamName, League, Division, Wins, Losses)
values
	('Cleveland', 'AL', 'Central', 91, 71),
	('Detroit', 'AL', 'Central', 64, 98),
	('Minnesota', 'AL', 'Central', 78, 84),
	('Chicago', 'AL', 'Central', 62, 100),
	('Kansas City', 'AL', 'Central', 58, 104),

	('New York', 'AL', 'East', 100, 62),
	('Boston', 'AL', 'East', 108, 54),
	('Tampa Bay', 'AL', 'East', 90, 72),
	('Toronto', 'AL', 'East', 73, 89),
	('Baltimore', 'AL', 'East', 47, 115),

	('Houston', 'AL', 'West', 103, 59),
	('Seattle', 'AL', 'West', 89, 73),
	('Los Angeles', 'AL', 'West', 80, 82),
	('Oakland', 'AL', 'West', 97, 65),
	('Texas', 'AL', 'West', 67, 95),

	('Milwaukee', 'NL', 'Central', 96, 67),
	('St. Louis', 'NL', 'Central', 88, 74),
	('Chicago', 'NL', 'Central', 95, 68),
	('Pittsburgh', 'NL', 'Central', 82, 79),
	('Cincinnati', 'NL', 'Central', 67, 95),

	('Atlanta', 'NL', 'East', 90, 72),
	('Philadelphia', 'NL', 'East', 80, 82),
	('New York', 'NL', 'East', 77, 85),
	('Washington', 'NL', 'East', 82, 80),
	('Miami', 'NL', 'East', 63, 98),

	('Colorado', 'NL', 'West', 91, 72),
	('Arizona', 'NL', 'West', 82, 80),
	('San Francisco', 'NL', 'West', 73, 89),
	('Los Angeles', 'NL', 'West', 92, 72),
	('San Diego', 'NL', 'West', 66, 96);
go

-- Compare ROW_NUMBER vs RANK vs DENSE_RANK.
select		s.TeamName,
			s.Wins,
			s.Losses,
			format(s.Wins * 1. / (s.Wins + s.Losses), '#.##0') Pct,
			row_number() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [row_number],
			rank()       over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [rank],
			dense_rank() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [dense_rank]
from		#BaseballStandings s
where		s.League = 'AL'
and			s.Division = 'Central'
order by	s.Wins desc;

-- Let's look at all of MLB.
-- This particular isn't very useful; we don't typically rank all of MLB this way.
select		s.TeamName,
			s.League,
			s.Division,
			s.Wins,
			s.Losses,
			format(s.Wins * 1. / (s.Wins + s.Losses), '#.##0') Pct,
			row_number() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [row_number],
			rank()       over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [rank],
			dense_rank() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [dense_rank]
from		#BaseballStandings s
order by	s.Wins desc;

-- Change the OVER clause to partition by league and division.
-- However, notice that we are doing ORDER BY only on Wins.
select		s.TeamName,
			s.League,
			s.Division,
			s.Wins,
			s.Losses,
			format(s.Wins * 1. / (s.Wins + s.Losses), '#.##0') Pct,
			row_number() over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [row_number],
			rank()       over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [rank],
			dense_rank() over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [dense_rank]
from		#BaseballStandings s
order by	s.Wins desc;

-- OK, this is better.
select		s.TeamName,
			s.League,
			s.Division,
			s.Wins,
			s.Losses,
			format(s.Wins * 1. / (s.Wins + s.Losses), '#.##0') Pct,
			row_number() over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [row_number],
			rank()       over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [rank],
			dense_rank() over (partition by s.League, s.Division order by s.Wins * 1. / (s.Wins + s.Losses) desc) [dense_rank]
from		#BaseballStandings s
order by	s.League, s.Division, s.Wins desc;





-- Ignore; used for display purposes only.
select		s.TeamName,
			s.Wins,
			s.Losses,
			format(s.Wins * 1. / (s.Wins + s.Losses), '#.##0') Pct,
			row_number() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [row_number],
			rank()       over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [rank],
			dense_rank() over (order by s.Wins * 1. / (s.Wins + s.Losses) desc) [dense_rank]
from		#BaseballStandings s
where		s.League = 'AL'
and			s.Division = 'Central'
union all
			select '',
			0,
			0,
			'.000',
			0,
			0,
			0
union all
			select '',
			0,
			0,
			'.000',
			0,
			0,
			0
order by	s.Wins desc;
